/*******************************************************************************
    Copyright (c) 2016 NVidia Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*******************************************************************************/


#ifndef _NVSWITCH_EXPORT_H_
#define _NVSWITCH_EXPORT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "nvlink_common.h"

#define NVSWITCH_DRIVER_NAME                            "nvswitch"
#define NVSWITCH_DEVICE_NAME                            "NVSwitch Device"
#define NVSWITCH_LINK_NAME                              "NVSwitch Link"
#define NVSWITCH_MAX_BARS                               1

#define NVSWITCH_DEVICE_INSTANCE_MAX                    64

#ifndef PCI_CLASS_BRIDGE_SVNP01
#define PCI_CLASS_BRIDGE_SVNP01                         0x0680
#endif

#ifndef PCI_DEVICE_ID_SVNP01
#define PCI_DEVICE_ID_SVNP01                            0x10F5
#endif

#ifndef PCI_VENDOR_ID_NVIDIA
#define PCI_VENDOR_ID_NVIDIA                            0x10DE
#endif

#ifndef PCI_REVISION_ID_SVNP01
#define PCI_REVISION_ID_SVNP01                          0x0
#endif


/*
 * @Brief : Initializes and registers the NvSwitch driver with NVlink.
 *
 * @Description :
 *
 * @param[in] accepted_domain   - Accepted NvSwitch domain. Links which
 *                                  appear on domains other than this number
 *                                  will be ignored, unless the accepted domain
 *                                  is 0xFFFFFFFF.
 * @param[in] accepted_link_mask - Mask of accepted links. Link indices whose bits
 *                                  are not raised in this mask will be ignored.
 *
 * @returns                 NVL_SUCCESS if action succeeded,
 *                          an NVL error code otherwise
 */
NvlStatus NVLINK_API_CALL  nvswitch_lib_initialize
(
    NvU32 accepted_domain,
    NvU32 accepted_link_mask
);

/*
 * @Brief : Shuts down and unregisters the driver/devices from the NVlink
 *              library.
 *
 * @Description :
 *
 * @returns                 NVL_SUCCESS if the action succeeded
 *                          an NVL error code otherwise
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_shutdown(void);

/*
 * @Brief : Creates and registers a device with the given data with the nvlink
 *              core library.
 *
 * @Description :
 *
 * @param[in] domain        pci domain of the device
 * @param[in] bus           pci bus of the device
 * @param[in] device        pci device of the device
 * @param[in] func          pci function of the device
 * @param[in] os_handle     Device handle used to interact with OS layer
 * @param[in] os_instance   instance number of this device
 *
 * @returns                 NVL_SUCCESS if the action succeeded
 *                          an NVL error code otherwise
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_register_device
(
    NvU16 domain, 
    NvU8 bus, 
    NvU8 device, 
    NvU8 func,
    void *os_handle,
    NvU32 os_instance
);

/*
 * @Brief : Unregister and clean-up a device with the given data with the nvlink
 *              core library.
 *
 * @Description :
 *
 * @param[in] domain        pci domain of the device
 * @param[in] bus           pci bus of the device
 * @param[in] device        pci device of the device
 * @param[in] func          pci function of the device
 *
 * @returns                 none
 */
void NVLINK_API_CALL nvswitch_lib_unregister_device
(
    NvU16 domain, 
    NvU8 bus, 
    NvU8 device, 
    NvU8 func
);

/*
 * @Brief: NvLink ctrl parameter structure
 * cmd              - Enumerated command to execute.
 * params           - Params structure to pass to the command.
 * params_size      - Size of the parameter structure.
 * handle           - Client handle to use for the command.
 */
typedef struct nvswitch_ctrl_params
{
    NvU32                   cmd;            // command #
    void                   *params;         // Client-specified params
    NvU64                   params_size;    // Client params size (bytes)
    void                   *handle;         // Client device handle
} NVSWITCH_CTRL_PARAMS;

/* 
 * @Brief Internal Control call interface.
 * Platform-specific control interfaces should route through here.
 * Crafts the internal control params structure from provided arguments
 * and passes them to control call implementation.
 *
 * @param[in] params        Client parameters specifying command to run
 *
 * @return                  NVL_SUCCESS on a successful command
 *                          -NVL_NOT_FOUND if target device unable to be found
 *                          -NVL_BAD_ARGS if an invalid cmd is provided
 *                          -NVL_BAD_ARGS if a null arg is provided
 *                          -NVL_ERR_GENERIC otherwise
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_ctrl
(
    NVSWITCH_CTRL_PARAMS *params
);

/*
 * @Brief : Retrieve OS handle for device instance
 *
 * @Description :
 *
 * @param[in] os_instance   device instance
 * @param[out] os_handle    OS handle registered for requested instance
 *
 * @returns                 NVL_SUCCESS if the action succeeded
 *                          -NVL_BAD_ARGS if bad arguments provided
 *                          -NVL_NOT_FOUND if index has not been registered
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_get_os_handle_by_instance(NvU32 os_instance, void **os_handle);

/*
 * @Brief : Services an interrupt triggered for the device with the given info.
 *
 * @Description :
 *
 * @param[in] info          reference to the pci info of the device to service
 * @param[in] reenable      re-eneable interrupts after processing is done
 *
 * @returns                 NVL_SUCCESS if there were no errors and interrupts were handled
 *                          -NVL_BAD_ARGS if bad arguments provided
 *                          -NVL_PCI_ERROR if there was a PCI error
 *                          -NVL_MORE_PROCESSING_REQUIRED no interrupts were found for this device
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_service_device(nvlink_pci_info *info, NvBool reenable);


/*
 * Initializes nvswitch library, preparing the driver to register
 *     discovered devices into the core library.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_load
(
    NvU32 accepted_domain,
    NvU32 accepted_link_mask
);

/*
 * Initializes the pci bus for the given device, including
 *     enabling device memory transactions and bus mastering.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_initialize_device_pci_bus(void *handle);

/*
 * Maps the device base address registers into CPU memory, and
 *     populates the device pci data with the mapping.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_initialize_device_bar_info(void *handle, nvlink_pci_info *info);

/*
 * Registers an interrupt service routine with the operating system
 *     to handle device interrupts.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_initialize_device_interrupt
(
    void *handle,
    nvlink_pci_info *info
);

/*
 * Shuts down the nvswitch library, deregistering its devices from
 *     the core and freeing core operating system accounting info.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_unload(void);

/*
 * Unmaps the previously mapped base address registers from cpu memory.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_shutdown_device_bar_info(void *handle, nvlink_pci_info *info);

/*
 * Unregisters the interrupt service routine from the operating system.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_shutdown_device_interrupt
(
    void *handle,
    nvlink_pci_info *info
);

/*
 * Cleans up any state the arch layer allocated for this device.
 */
NvlStatus NVLINK_API_CALL nvswitch_lib_release_device(void *handle);

/*
 * Detects failure condition on the requested device.
 */
NvBool NVLINK_API_CALL nvswitch_lib_check_failure(void *handle);

#ifdef __cplusplus
}
#endif

#endif //_NVSWITCH_EXPORT_H_
